"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupWindowsEnvironmentPath = void 0;
const error_1 = __importDefault(require("@pnpm/error"));
const path_1 = require("path");
const execa_1 = __importDefault(require("execa"));
const string_prototype_matchall_1 = __importDefault(require("string.prototype.matchall"));
const errors_1 = require("./errors");
const REG_KEY = 'HKEY_CURRENT_USER\\Environment';
async function setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    var _a;
    // Use `chcp` to make `reg` use utf8 encoding for output.
    // Otherwise, the non-ascii characters in the environment variables will become garbled characters.
    const chcpResult = await (0, execa_1.default)('chcp');
    const cpMatch = (_a = /\d+/.exec(chcpResult.stdout)) !== null && _a !== void 0 ? _a : [];
    const cpBak = parseInt(cpMatch[0]);
    if (chcpResult.failed || !(cpBak > 0)) {
        return `exec chcp failed: ${cpBak}, ${chcpResult.stderr}`;
    }
    await (0, execa_1.default)('chcp', ['65001']);
    try {
        return await _setupWindowsEnvironmentPath(path_1.win32.normalize(pnpmHomeDir), opts);
    }
    finally {
        await (0, execa_1.default)('chcp', [cpBak.toString()]);
    }
}
exports.setupWindowsEnvironmentPath = setupWindowsEnvironmentPath;
async function _setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    const registryOutput = await getRegistryOutput();
    const logger = [];
    logger.push(logEnvUpdate(await updateEnvVariable(registryOutput, 'PNPM_HOME', pnpmHomeDir, opts), 'PNPM_HOME'));
    logger.push(logEnvUpdate(await prependToPath(registryOutput, '%PNPM_HOME%'), 'Path'));
    return logger.join('\n');
}
function logEnvUpdate(envUpdateResult, envName) {
    switch (envUpdateResult) {
        case 'skipped': return `${envName} was already up-to-date`;
        case 'updated': return `${envName} was updated`;
    }
    return '';
}
async function updateEnvVariable(registryOutput, name, value, opts) {
    const currentValue = await getEnvValueFromRegistry(registryOutput, name);
    if (currentValue && !opts.force) {
        if (currentValue !== value) {
            throw new errors_1.BadEnvVariableError({ envName: name, currentValue, wantedValue: value });
        }
        return 'skipped';
    }
    else {
        await setEnvVarInRegistry(name, value);
        return 'updated';
    }
}
async function prependToPath(registryOutput, prependDir) {
    const pathData = await getEnvValueFromRegistry(registryOutput, 'Path');
    if (pathData === undefined || pathData == null || pathData.trim() === '') {
        throw new error_1.default('NO_PATH', '"Path" environment variable is not found in the registry');
    }
    else if (pathData.split(path_1.win32.delimiter).includes(prependDir)) {
        return 'skipped';
    }
    else {
        const newPathValue = `${prependDir}${path_1.win32.delimiter}${pathData}`;
        await setEnvVarInRegistry('Path', newPathValue);
        return 'updated';
    }
}
// `windowsHide` in `execa` is true by default, which will cause `chcp` to have no effect.
const EXEC_OPTS = { windowsHide: false };
/**
 * We read all the registry values and then pick the keys that we need.
 * This is done because if we would try to pick a key that is not in the registry, the command would fail.
 * And it is hard to identify the real cause of the command failure.
 */
async function getRegistryOutput() {
    try {
        const queryResult = await (0, execa_1.default)('reg', ['query', REG_KEY], EXEC_OPTS);
        return queryResult.stdout;
    }
    catch (err) { // eslint-disable-line
        throw new error_1.default('REG_READ', 'win32 registry environment values could not be retrieved');
    }
}
async function getEnvValueFromRegistry(registryOutput, envVarName) {
    const regexp = new RegExp(`^ {4}(?<name>${envVarName}) {4}(?<type>\\w+) {4}(?<data>.*)$`, 'gim');
    const match = Array.from((0, string_prototype_matchall_1.default)(registryOutput, regexp))[0];
    return match === null || match === void 0 ? void 0 : match.groups.data;
}
async function setEnvVarInRegistry(envVarName, envVarValue) {
    try {
        await (0, execa_1.default)('reg', ['add', REG_KEY, '/v', envVarName, '/t', 'REG_EXPAND_SZ', '/d', envVarValue, '/f'], EXEC_OPTS);
    }
    catch (err) { // eslint-disable-line
        throw new error_1.default('FAILED_SET_ENV', `Failed to set "${envVarName}" to "${envVarValue}": ${err.stderr}`);
    }
    // When setting environment variables through the registry, they will not be recognized immediately.
    // There is a workaround though, to set at least one environment variable with `setx`.
    // We have some redundancy here because we run it for each env var.
    // It would be enough also to run it only for the last changed env var.
    // Read more at: https://bit.ly/39OlQnF
    await (0, execa_1.default)('setx', [envVarName, envVarValue], EXEC_OPTS);
}
//# sourceMappingURL=setupOnWindows.js.map